@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 Ed Schouten <ed@fxq.nl>
+ * Copyright (c) 2006-2008 Ed Schouten <ed@80386.nl>
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -49,11 +49,15 @@ static struct vfsref *vr_curdir = NULL;
  * @brief List we use when displaying a file instead of a directory,
  *        containing only the file itself.
  */
-static struct vfslist vl_flist;
+static struct vfslist vl_flist = VFSLIST_INITIALIZER;
 /**
  * @brief Reference to window used as the file browser.
  */
 static struct gui_vfslist *win_browser;
+/**
+ * @brief The current filtering string that's being applied.
+ */
+static char *locatestr = NULL;
 
 /**
  * @brief Refresh the bar above the filebrowser to contain the proper
@@ -70,6 +74,13 @@ gui_browser_dirname_refresh(void)
 	werase(win_dirname);
 	if (vr_curdir != NULL) {
 		mvwaddstr(win_dirname, 0, 1, vfs_filename(vr_curdir));
+		if (locatestr != NULL) {
+			waddstr(win_dirname, " (");
+			waddstr(win_dirname, _("filter"));
+			waddstr(win_dirname, ": ");
+			waddstr(win_dirname, locatestr);
+			waddstr(win_dirname, ")");
+		}
 	}
 
 	percent = gui_vfslist_getpercentage(win_browser);
@@ -80,6 +91,24 @@ gui_browser_dirname_refresh(void)
 	gui_unlock();
 }
 
+/**
+ * @brief Clean up our pseudo-directory data.
+ */
+static void
+gui_browser_cleanup_flist(void)
+{
+	struct vfsref *vr;
+
+	/* Close our fake view */
+	while ((vr = vfs_list_first(&vl_flist)) != NULL) {
+		vfs_list_remove(&vl_flist, vr);
+		vfs_close(vr);
+	}
+
+	g_free(locatestr);
+	locatestr = NULL;
+}
+
 void
 gui_browser_init(void)
 {
@@ -101,11 +130,11 @@ gui_browser_init(void)
 	defdir = config_getopt("gui.browser.defaultpath");
 	if (defdir[0] != '\0') {
 		/* Open predefined directory */
-		vr_curdir = vfs_open(defdir, NULL, NULL, 0);
+		vr_curdir = vfs_lookup(defdir, NULL, NULL, 0);
 	} else {
 		/* Open current directory */
 		cwd = g_get_current_dir();
-		vr_curdir = vfs_open(cwd, NULL, NULL, 1);
+		vr_curdir = vfs_lookup(cwd, NULL, NULL, 1);
 		g_free(cwd);
 	}
 
@@ -124,6 +153,9 @@ gui_browser_destroy(void)
 {
 	delwin(win_dirname);
 	gui_vfslist_destroy(win_browser);
+
+	/* Clean up our mess */
+	gui_browser_cleanup_flist();
 	if (vr_curdir != NULL)
 		vfs_close(vr_curdir);
 }
@@ -186,31 +218,30 @@ gui_browser_cursor_tail(void)
  */
 
 void
-gui_browser_dir_parent(void)
+gui_browser_gotofile(struct vfsref *vr)
 {
-	struct vfsref *vr, *vrn;
+	struct vfsref *vrp, *vrn;
 	unsigned int idx;
 
-	if (vr_curdir == NULL)
-		return;
-
-	if ((vr = vfs_open("..", NULL, vfs_filename(vr_curdir), 1)) == NULL)
+	if ((vrp = vfs_lookup("..", NULL, vfs_filename(vr), 1)) == NULL)
 		goto bad;
-	if (vfs_populate(vr) != 0) {
+	if (vfs_populate(vrp) != 0) {
 		/* Permission denied? */
-		vfs_close(vr);
+		vfs_close(vrp);
 		goto bad;
 	}
 	
-	for (vrn = vfs_list_first(vfs_population(vr)), idx = 1;
+	for (vrn = vfs_list_first(vfs_population(vrp)), idx = 1;
 	    vrn != NULL; vrn = vfs_list_next(vrn), idx++) {
 		/* Select the previous directory */
-		if (strcmp(vfs_name(vr_curdir), vfs_name(vrn)) == 0)
+		if (strcmp(vfs_name(vr), vfs_name(vrn)) == 0)
 			break;
 	}
 
+	/* Change the directory */
+	gui_browser_cleanup_flist();
 	vfs_close(vr_curdir);
-	vr_curdir = vr;
+	vr_curdir = vrp;
 	gui_vfslist_setlist(win_browser, vfs_population(vr_curdir));
 
 	if (vrn != NULL)
@@ -221,15 +252,32 @@ bad:
 	gui_msgbar_warn(_("Unable to enter the parent directory."));
 }
 
+void
+gui_browser_dir_parent(void)
+{
+
+	if (vr_curdir == NULL)
+		return;
+	
+	if (locatestr != NULL) {
+		/* First unset the filter if we have one */
+		gui_browser_cleanup_flist();
+		gui_vfslist_setlist(win_browser, vfs_population(vr_curdir));
+		return;
+	}
+
+	gui_browser_gotofile(vr_curdir);
+}
+
 void
 gui_browser_dir_enter(void)
 {
 	struct vfsref *vr;
 
-	if ((vr = gui_vfslist_getselected(win_browser)) == NULL)
+	if (gui_vfslist_warn_isempty(win_browser))
 		return;
 
-	vr = vfs_dup(vr);
+	vr = vfs_dup(gui_vfslist_getselected(win_browser));
 	if (vfs_populate(vr) != 0) {
 		if (vfs_populatable(vr)) {
 			/* Permission denied? */
@@ -240,6 +288,8 @@ gui_browser_dir_enter(void)
 		return;
 	}
 
+	/* Change the directory */
+	gui_browser_cleanup_flist();
 	vfs_close(vr_curdir);
 	vr_curdir = vr;
 	gui_vfslist_setlist(win_browser, vfs_population(vr_curdir));
@@ -302,9 +352,9 @@ gui_browser_playq_add_before(void)
 }
 
 int
-gui_browser_searchnext(const regex_t *match)
+gui_browser_searchnext(const struct vfsmatch *vm)
 {
-	return gui_vfslist_searchnext(win_browser, match);
+	return gui_vfslist_searchnext(win_browser, vm);
 }
 
 void
@@ -323,10 +373,10 @@ gui_browser_chdir(void)
 
 	if (vr_curdir != NULL) {
 		/* Relative to the current node */
-		vr = vfs_open(path, NULL, vfs_filename(vr_curdir), 0);
+		vr = vfs_lookup(path, NULL, vfs_filename(vr_curdir), 0);
 	} else {
 		/* Relative to the root */
-		vr = vfs_open(path, NULL, NULL, 0);
+		vr = vfs_lookup(path, NULL, NULL, 0);
 	}
 	g_free(path);
 
@@ -337,6 +387,7 @@ gui_browser_chdir(void)
 			goto bad;
 
 		/* Replace old directory */
+		gui_browser_cleanup_flist();
 		if (vr_curdir != NULL)
 			vfs_close(vr_curdir);
 		vr_curdir = vr;
@@ -347,8 +398,7 @@ gui_browser_chdir(void)
 			    vfs_population(vr));
 		} else {
 			/* Create a fake directory */
-			vfs_list_init(&vl_flist);
-			vfs_list_insert_tail(&vl_flist, vr);
+			vfs_list_insert_tail(&vl_flist, vfs_dup(vr));
 			gui_vfslist_setlist(win_browser, &vl_flist);
 		}
 
@@ -387,6 +437,7 @@ gui_browser_write_playlist(void)
 	vfs_populate(vr);
 
 	/* Replace old directory */
+	gui_browser_cleanup_flist();
 	if (vr_curdir != NULL)
 		vfs_close(vr_curdir);
 	vr_curdir = vr;
@@ -400,3 +451,34 @@ gui_browser_fullpath(void)
 {
 	gui_vfslist_fullpath(win_browser);
 }
+
+int
+gui_browser_locate(const struct vfsmatch *vm)
+{
+	struct vfslist vl = VFSLIST_INITIALIZER;
+
+	if (vr_curdir == NULL)
+		return (-1);
+
+	/* Perform a search on the query */
+	vfs_locate(&vl, vr_curdir, vm);
+	if (vfs_list_empty(&vl))
+		return (-1);
+	
+	gui_browser_cleanup_flist();
+	locatestr = g_strdup(vfs_match_value(vm));
+	vfs_list_move(&vl_flist, &vl);
+	gui_vfslist_setlist(win_browser, &vl_flist);
+
+	return (0);
+}
+
+void
+gui_browser_gotofolder(void)
+{
+
+	if (gui_vfslist_warn_isempty(win_browser))
+		return;
+
+	gui_browser_gotofile(gui_vfslist_getselected(win_browser));
+}
