@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 Ed Schouten <ed@fxq.nl>
+ * Copyright (c) 2006-2008 Ed Schouten <ed@80386.nl>
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -28,12 +28,10 @@
  * @brief Internal textual user interface routines.
  */
 
-#ifndef _GUI_INTERNAL_H_
-#define _GUI_INTERNAL_H_
-
 #include CURSES_HEADER
 
 struct vfsref;
+struct vfsmatch;
 
 /**
  * @brief Determine whether our terminal is black or white or color.
@@ -125,7 +123,7 @@ int gui_input_askyesno(const char *question);
  *        and return the user response.
  */
 char *gui_input_askstring(const char *question, const char *defstr,
-    const char *allowed);
+    int (*validator)(const char *str, char c));
 
 /*
  *  gui_msgbar
@@ -250,7 +248,7 @@ void gui_playq_song_select(void);
  * @brief Search for the next item matching gui_input_cursearch in the
  *        playlist.
  */
-int gui_playq_searchnext(const regex_t *str);
+int gui_playq_searchnext(const struct vfsmatch *vm);
 /**
  * @brief Focus or unfocus the playlist.
  */
@@ -270,6 +268,15 @@ void gui_playq_volume_up(void);
  */
 void gui_playq_volume_down(void);
 #endif /* BUILD_VOLUME */
+/**
+ * @brief Go to the directory containing the selected item.
+ */
+void gui_playq_gotofolder(void);
+/**
+ * @brief Go to the parent directory of the VFS reference and select the
+ *        item which shares the same filename.
+ */
+void gui_browser_gotofile(struct vfsref *vr);
 
 /*
  * gui_browser
@@ -347,7 +354,7 @@ void gui_browser_playq_add_before(void);
  * @brief Search for the next item matching gui_input_cursearch in the
  *        file browser.
  */
-int gui_browser_searchnext(const regex_t *str);
+int gui_browser_searchnext(const struct vfsmatch *vm);
 /**
  * @brief Change to a specified directory.
  */
@@ -365,5 +372,11 @@ void gui_browser_write_playlist(void);
  *        bar.
  */
 void gui_browser_fullpath(void);
-
-#endif /* !_GUI_INTERNAL_H_ */
+/**
+ * @brief Apply a recursive search filter on the current directory.
+ */
+int gui_browser_locate(const struct vfsmatch *vm);
+/**
+ * @brief Go to the directory containing the selected item.
+ */
+void gui_browser_gotofolder(void);
