@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 Ed Schouten <ed@fxq.nl>
+ * Copyright (c) 2006-2008 Ed Schouten <ed@80386.nl>
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -28,15 +28,12 @@
  * @brief Playlist handling.
  */
 
-#ifndef _PLAYQ_H_
-#define _PLAYQ_H_
-
 struct vfsref;
 
 /**
  * @brief Initialize the playlist locking.
  */
-void playq_init(int xmms);
+void playq_init(int autoplay, int xmms, int load_dumpfile);
 /**
  * @brief Spawn the playback thread.
  */
@@ -157,6 +154,4 @@ void playq_song_fast_move_tail(struct vfsref *vr, unsigned int index);
 /**
  * @brief Switch playback to a specific song.
  */
-void playq_song_fast_select(struct vfsref *vr, unsigned int index);
-
-#endif /* !_PLAYQ_H_ */
+void playq_song_fast_select(struct vfsref *vr);
